import boto3
import time
import os

import psycopg2
import sys
from datetime import datetime, timedelta
import requests
from psc_orgkey import decode_orgkey

print("Initializing the variables")
# initialize the variables from environment variables
db_host = os.environ.get("RDS_HOSTNAME")
db_port = int(os.environ.get("RDS_PORT"))
db_user = os.environ.get("RDS_USERNAME")
db_pass = os.environ.get("RDS_PASSWORD")
db_name = os.environ.get("RDS_DB")
default_backend_url = os.environ.get("EXTERNAL_INGRESS_URL")
backend_urls = {
    "prod01": os.environ.get("PROD01_EXTERNAL_INGRESS_URL", "https://dashboard.confer.net"),
    "prod02": os.environ.get("PROD02_EXTERNAL_INGRESS_URL", "https://defense.conferdeploy.net"),
    "prod05": os.environ.get("PROD05_EXTERNAL_INGRESS_URL", "https://defense-prod05.conferdeploy.net"),
    "eap01": os.environ.get("EAP01_EXTERNAL_INGRESS_URL", "https://defense-eap01.conferdeploy.net"),
    "default": default_backend_url
}
default_auth_token = os.environ.get("DEFAULT_JOB_CONNECTOR",
                                    f"{os.environ.get('API_KEY')}"
                                    f"/{os.environ.get('CONNECTOR_ID')}")
auth_tokens = {
    "prod01": os.environ.get("PROD01_JOB_CONNECTOR"),
    "prod02": os.environ.get("PROD02_JOB_CONNECTOR"),
    "prod05": os.environ.get("PROD05_JOB_CONNECTOR"),
    "eap01": os.environ.get("EAP01_JOB_CONNECTOR"),
    "default": default_auth_token
}

print("Initializing the db connection")
# establish a connect with the postgres instance
try:
    conn = psycopg2.connect("dbname='%s' user='%s' host='%s' password='%s' port=%d"
                            % (db_name, db_user, db_host, db_pass, db_port))
except Exception as e:
    print("ERROR: Unexpected error: Could not connect to postgres instance. {}".format(e))
    sys.exit()


# function to get the unique orgs that might need cleanup
def get_unique_org_keys():
    epoch = int((datetime.now() - timedelta(days=1)).timestamp())
    query = "select distinct(org_key) from policyservice.rc_archived_parameter_values where created_at <= {} limit 1000" \
        .format(str(epoch))

    cursor = conn.cursor()
    cursor.execute(query)
    org_keys = [r[0] for r in cursor.fetchall()]
    return org_keys


# function to get the retention policy for a given org
def get_org_retention_policy_in_days(org_key):
    retention_in_days = 30

    headers = {"x-auth-token": get_auth_token(org_key)}
    entitlement_endpoint = "{}/entitlements/v1/orgs/{}/features".format(get_backend_url(org_key), org_key)
    response = requests.get(entitlement_endpoint, headers=headers)
    if response.ok:
        features = response.json()
        for feature in features:
            if "feature_id" in feature and feature["feature_id"] == "psc:feature:retention":
                if "custom_attr" in feature and "retention_days" in feature["custom_attr"]:
                    retention_in_days = feature["custom_attr"]["retention_days"]
    else:
        print("Error getting the retention policy for org {}. Error - {}".format(org_key, response.text))
        raise Exception("Unable to get the retention policy")

    print("Using retention days {} for org {}".format(retention_in_days, org_key))
    return retention_in_days


# function to purge archived user parameters for a given org, older than specified days
def purge_records_for_org_older_than_days(org_key, days):
    print("Purging records for org {}".format(org_key))
    epoch = int((datetime.now() - timedelta(days=days)).timestamp())
    get_purgeable_records_query = "select id from policyservice.rc_archived_parameter_values " \
                                  "where org_key = '{}' and created_at <= {}".format(org_key, epoch)

    cursor = conn.cursor()
    cursor.execute(get_purgeable_records_query)
    ids = [int(r[0]) for r in cursor.fetchall()]

    if ids is not None and len(ids) > 0:
        # delete the lookup values
        delete_lookups_query = 'delete from policyservice.rc_archived_parameter_value_lookup ' \
                               'where archived_param_values_id in %(ids)s'
        cursor = conn.cursor()
        cursor.execute(delete_lookups_query, {'ids': tuple(ids)})

        # delete parameter values
        delete_params_query = 'delete from policyservice.rc_archived_parameter_values where id in %(ids)s'
        cursor = conn.cursor()
        cursor.execute(delete_params_query, {'ids': tuple(ids)})
        conn.commit()


# Function to get the environment details for the given org
def get_org_env(org_key):
    try:
        dec = decode_orgkey(org_key)
        print("Environment for org key {} is {}".format(org_key, dec.env))
        return dec.env
    except Exception as e:
        print("Error decoding org key {}".format(e))
        return "default"


# Function to get the environment backend url for the given org
def get_backend_url(org_key):
    env = get_org_env(org_key)
    if env in backend_urls:
        return backend_urls.get(env)
    else:
        return backend_urls.get("default")


# Function to get the lambda auth token for the given org
def get_auth_token(org_key):
    env = get_org_env(org_key)
    if env in auth_tokens:
        return auth_tokens.get(env)
    else:
        return auth_tokens.get("default")


def processor():
    org_keys = get_unique_org_keys()
    print("Processing {} orgs".format(len(org_keys)))

    for org_key in org_keys:
        print("====================================================================================================")
        print("Processing org {}".format(org_key))
        try:
            retention_in_days = get_org_retention_policy_in_days(org_key)
            purge_records_for_org_older_than_days(org_key, retention_in_days)
        except Exception as e:
            print("Skipping org {}, error - {}".format(org_key, e))
        print("====================================================================================================")


if __name__ == '__main__':
    processor()
